@@ -17,6 +17,6 @@ def selenium_support(spider_cls):
 
 
 # 便于其它文件导入
-from .driver import Webdriver
+from .browser import Browser
 from .request import SeleniumRequest
 from .middleware import SeleniumDownloadMiddleWare, SeleniumNoBlockingDownloadMiddleWare
