@@ -3,9 +3,10 @@ import logging
 
 from scrapy import signals
 from scrapy.http import HtmlResponse
+from scrapy.http import Request, Response
 from selenium.webdriver.support.ui import WebDriverWait
 
-from scrapy_ajax_utils.selenium.driver import Webdriver
+from scrapy_ajax_utils.selenium.browser import Browser
 from scrapy_ajax_utils.selenium.request import SeleniumRequest
 from scrapy_ajax_utils.utils import extract_domain_from_url
 
@@ -21,22 +22,22 @@ class SeleniumDownloadMiddleWare(object):
         否则某些网站可能会对此做验证 导致 cookies 无效
     """
 
-    def __init__(self, creator):
-        self.creator = creator
+    def __init__(self, browser):
+        self.browser = browser
         self.cached_cookies = {}
         self._driver = None
 
     @property
     def driver(self):
         if self._driver is None:
-            self._driver = self.creator.driver()
+            self._driver = self.browser.driver()
         return self._driver
 
     @classmethod
     def from_crawler(cls, crawler):
         settings = crawler.settings
-        dm = cls(_make_driver_creator_from_settings(settings))
-        crawler.signals.connect(dm.closed, signal=signals.spider_closed)
+        dm = cls(_make_browser_from_settings(settings))
+        crawler.signals.connect(dm.close, signal=signals.spider_closed)
         return dm
 
     def process_request(self, request, spider):
@@ -75,34 +76,27 @@ class SeleniumDownloadMiddleWare(object):
             request.meta['js_result'] = driver.execute_script(request.script)
 
         if request.handler:
-            request.handler(driver, request, spider)
+            handle_result = request.handler(driver, request, spider)
+        else:
+            handle_result = None
 
-        for cookie_name, cookie_value in request.cookies.items():
-            driver.add_cookie(
-                {
-                    'name': cookie_name,
-                    'value': cookie_value
-                }
-            )
         request.cookies = driver.get_cookies()
-        request.meta['driver'] = driver
-
         if request.cache_cookies:
             domain = extract_domain_from_url(request.url)
             self.cached_cookies[domain] = request.cookies
-        else:
-            body = str.encode(driver.page_source)
-            return HtmlResponse(
-                driver.current_url,
-                body=body,
-                encoding='utf-8',
-                request=request
-            )
 
-    def closed(self):
+        if isinstance(handle_result, (Request, Response)):
+            return handle_result
+
+        return HtmlResponse(driver.current_url,
+                            body=str.encode(driver.page_source),
+                            encoding='utf-8',
+                            request=request)
+
+    def close(self):
         if self._driver is not None:
             self._driver.quit()
-            logger.debug('Selenium closed')
+            logger.debug('Selenium close')
 
 
 class SeleniumNoBlockingDownloadMiddleWare(SeleniumDownloadMiddleWare):
@@ -115,57 +109,61 @@ class SeleniumNoBlockingDownloadMiddleWare(SeleniumDownloadMiddleWare):
     @classmethod
     def from_crawler(cls, crawler):
         max_drivers = crawler.settings.get('SELENIUM_MAX_DRIVERS', 5)
-        dm = cls(_make_driver_creator_from_settings(crawler.settings), max_drivers)
-        crawler.signals.connect(dm.closed, signal=signals.spider_closed)
+        dm = cls(_make_browser_from_settings(crawler.settings), max_drivers)
+        crawler.signals.connect(dm.close, signal=signals.spider_closed)
         return dm
 
-    def __init__(self, creator, max_drivers):
-        super().__init__(creator)
+    def __init__(self, browser, max_drivers):
+        super().__init__(browser)
         self.max_drivers = max_drivers
-        self.loop = asyncio.get_event_loop()
         self.launched_drivers = []
+        self.loop = asyncio.get_event_loop()
 
     async def process_request(self, request, spider):
         if not isinstance(request, SeleniumRequest) or self.has_cached_cookies(request):
             return
 
-        async with self.lock:
-            if len(self.launched_drivers) < self.max_drivers:
-                self.launched_drivers.append(DriverItem(self.creator.driver()))
-
-        for driver_item in self.launched_drivers:
-            if driver_item.is_idle:
-                driver_item.is_idle = False
-                response = await self.loop.run_in_executor(None, self.process_by_driver, request, spider, driver_item.driver)
-                driver_item.is_idle = True
-                return response
-        else:
+        driver = await self.get_idle_driver()
+        if not driver:
             # Maybe next time ..
             request.dont_filter = True
             return request
 
-    def closed(self):
-        for driver_item in self.launched_drivers:
-            driver_item.driver.quit()
-        super().closed()
+        response = await self.loop.run_in_executor(None, self.process_by_driver, request, spider, driver)
+        driver.set_idle()
+        return response
+
+    async def get_idle_driver(self):
+        async with self.lock:
+            for driver in self.launched_drivers:
+                if driver.is_idle:
+                    driver.set_busy()
+                    return driver
+
+            if len(self.launched_drivers) < self.max_drivers:
+                driver = await self.loop.run_in_executor(None, self.launch_driver)
+                return driver
+
+    def launch_driver(self):
+        driver = self.browser.driver()
+        self.launched_drivers.append(driver)
+        return driver
+
+    def close(self):
+        for driver in self.launched_drivers:
+            driver.quit()
+        super().close()
 
 
-class DriverItem(object):
-
-    def __init__(self, driver):
-        self.driver = driver
-        self.is_idle = True
-
-
-def _make_driver_creator_from_settings(settings):
+def _make_browser_from_settings(settings):
     headless = settings.getbool('SELENIUM_HEADLESS', True)
     disable_image = settings.get('SELENIUM_DISABLE_IMAGE', True)
     driver_name = settings.get('SELENIUM_DRIVER_NAME', 'chrome')
     executable_path = settings.get('SELENIUM_DRIVER_PATH')
     user_agent = settings.get('USER_AGENT')
-    creator = Webdriver(headless=headless,
-                        disable_image=disable_image,
-                        driver_name=driver_name,
-                        executable_path=executable_path,
-                        user_agent=user_agent)
+    creator = Browser(headless=headless,
+                      disable_image=disable_image,
+                      driver_name=driver_name,
+                      executable_path=executable_path,
+                      user_agent=user_agent)
     return creator
